import React, { useState, useEffect, useRef, useCallback } from 'react';
import { getAvailability, setAvailability } from './api';


// adapted from https://stackoverflow.com/a/60316951/13917490
const START_DATE = new Date('05/30/2021');
const END_DATE = new Date('09/01/2021');
const DATES = Array.from(
  { length: ((END_DATE - START_DATE) / 864e5) + 1 },
  (_, i) => {
    const date = new Date(START_DATE);
    date.setDate(date.getDate()+i);
    return { month: date.getMonth() + 1, day: date.getDate() };
  }
);

const NameForm = ({ onNameSet }) => {
  const [name, setName] = useState("");

  return (
    <div
      style={{
        display: "flex",
        flexFlow: "column nowrap",
      }}
    >
      <span>Vacation Planner</span>
      <span>Enter the name you want to use for the planner</span>
      <span>Remember the name you use b/c you'll need it to log back in at a later time</span>
      <span>Please just behave and use a reasonable name</span>
      <span>I spend my day job cleaning up edge cases from bad users</span>
      <form onSubmit={() => onNameSet(name.trim())}>
        <input 
          type="text" 
          onChange={({ target }) => setName(target.value)}
          value={name}
        />
        <input 
          disabled={name === ""} 
          type="submit" 
          value="Submit"
        />
      </form>
    </div>
  )
}

const colors = {
  yes: "#0f0",
  no: "#f00",
  maybe: "#ff0",
}

const getNext = (user, availability) => ({
  yes: "no",
  no: "maybe",
  maybe: "unknown",
  unknown: "yes",
})[availability.find(({ name }) => name === user)?.status ?? "unknown"];

const pad = num => {
  let snum = "" + num
  if (snum.length === 1) snum = "0" + snum
  return snum
}

const Tile = ({ user, month, day, availability, onClick }) => (
  <div 
    onClick={() => onClick(month, day, getNext(user, availability))}
    style={{
      display: "flex",
      flexFlow: "column nowrap",
      width: "10em",
      height: "10em",
      border: "1px solid black",
      margin: "2px 2px 2px 2px",
      backgroundColor: colors[availability.filter(({ name }) => name === user)[0]?.status] || "#fff",
    }}
  >
    <span>{pad(month)}/{pad(day)}</span>
    {
      availability
        .filter(({ status }) => status && status !== "unknown")
        .filter(({ name }) => name !== user)
        .map(({ name, status }) => (
          <span 
            key={name} 
            style={{ 
              backgroundColor: colors[status],
              overflow: "hidden",
              textOverflow: "ellipsis",
            }}
          >
            {name}
          </span>
        ))
      }
  </div>
);


const chunk = (arr, len) => {
  const chunks = [];
  for (let i = 0; i < arr.length;) {
    chunks.push(arr.slice(i, i += len));
  }
  return chunks;
}

const DaysOfWeek = () => (
  <div
    style={{
      display: "flex",
      flexFlow: "row nowrap",
    }}
  >
    {
      ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
      .map(day => (
        <div
          key={day}
          style={{
            width: "10em",
            height: "1.5em",
            border: "1px solid black",
            margin: "2px 2px 2px 2px",
          }}
        >
          <span style={{ paddingLeft: "0.5em" }}>{day}</span>
        </div>
      ))
    }
  </div>
)

const Header = ({ name }) => (
  <>
    <span>Ignore how ugly this site is I wrote it in a total of 10 hours</span>
    <span>Logged in as: {name}</span>
    <span>Green is definitely available</span>
    <span>Red is definitely not available</span>
    <span>Yellow is possibly available</span>
    <span>White is unknown availability</span>
    <span>WEEK buttons toggle a whole week</span>
    <DaysOfWeek />
  </>
)

const Week = ({ days, user, change }) => (
  <div
    style={{
      display: "flex",
      flexFlow: "row nowrap",
      alignItems: "center",
    }}
  >
    <div 
      style={{
        display: "flex",
        flexFlow: "row nowrap",
      }}
    >
      {
        days.map(item => (
          <Tile
            key={item.day} 
            user={user}
            month={item.month}
            day={item.day}
            availability={item.availability}
            onClick={(month, day, status) => change(user, [{ month, day, status }])}
          />
        ))
      }
    </div>
    <div
      onClick={() => change(user, days.map(({ month, day, availability }) => ({
        month,
        day,
        status: getNext(user, availability),
      })))}
      style={{
        paddingLeft: "1em",
      }}
    >
      WEEK
    </div>
  </div>
)

const useCalendar = () => {
  const [calendar, setCalendar] = useState(null);
  const lastUpdatedRef = useRef(null);
  const updateCalendar = useCallback(async () => {
    const { availability, lastUpdated } = await getAvailability();
    if (lastUpdated !== lastUpdatedRef.current) {
      // only actually update if there are changes
      lastUpdatedRef.current = lastUpdated;
      const newCalendar = new Map();
      availability.forEach(({ month, day, availability: availabilityList }) => {
        newCalendar.set(`${month}-${day}`, availabilityList);
      });
      setCalendar(newCalendar);
    }
  }, [lastUpdatedRef]);

  useEffect(() => {
    // do the first update
    updateCalendar();
    // update every few seconds
    const interval = setInterval(updateCalendar, 5000);
    return () => { clearInterval(interval); };
  }, [updateCalendar]);

  const getter = (month, day) => calendar?.get(`${month}-${day}`) ?? [];

  const change = useCallback(async (name, updates) => {
    await setAvailability(name, updates);
    await updateCalendar();
  }, [updateCalendar]);

  return [getter, change];
}

function App() {
  const [name, setName] = useState(null);
  const [getDay, change] = useCalendar();

  if (name === null) { 
    return <NameForm onNameSet={setName} />; 
  }


  return (
    <div
      style={{
        display: "flex",
        flexFlow: "column nowrap",
      }}
    >
      <Header name={name} />
      {
        chunk(DATES, 7).map(days => 
          <Week 
            key={`${days[0].month}-${days[0].day}`} 
            days={days.map(({ month, day }) => ({ month, day, availability: getDay(month, day) }))} 
            change={change}
            user={name}
          />
        )
      }
    </div>
  );
}

export default App;
